import React, { useCallback, useState } from "react";
import { View, StyleSheet, TouchableOpacity } from "react-native";
import AntDesign from "react-native-vector-icons/AntDesign";
import MaterialCommunityIcons from "react-native-vector-icons/MaterialCommunityIcons";
import Feather from "react-native-vector-icons/Feather";
import { Card, Button, Text, ActivityIndicator } from "react-native-paper";
import { useField } from "formik";
import { addOrRemoveArr, formatFileSize } from "../../helpers";
import { FileDto } from "../../typings/file";
import { useActionSheet } from "@expo/react-native-action-sheet";
import { useLMS } from "../../context/context";
import ImagePicker, { Image, Options } from "react-native-image-crop-picker";
import { AwsResponseProp } from "../../typings";
import { showMessage } from "react-native-flash-message";

type Props = {
  name: string;
  disabled?: boolean;
  title: string;
  buttonLabel?: string;
};

const imageOption: Options = {
  multiple: false,
  cropping: true,
  mediaType: "photo",
  freeStyleCropEnabled: true,
};

export const UploadAttachment: React.FC<Props> = ({
  name,
  disabled,
  title,
  buttonLabel,
}) => {
  const [loading, setLoading] = useState(false);
  const [tempFile, setTempFile] = useState({ url: "", name: "" });
  const [, meta, helpers] = useField(name);

  const { axiosInstance } = useLMS();
  const { showActionSheetWithOptions } = useActionSheet();

  const uploadADoc = useCallback(
    async (fileName: string) => {
      try {
        const { data } = await axiosInstance.get(
          `/v1/external-lending/sme/s3/presigned-url?file_name=${fileName}`,
        );
        return data;
      } catch (error: any) {
        throw error.response.data;
      }
    },
    [axiosInstance],
  );

  const onSuccessfulUpload = useCallback(
    (uploadUrl: string, image: Image) => {
      helpers.setValue([
        ...meta.value,
        {
          name: image.filename,
          size: image.size,
          resourceURL: uploadUrl,
        },
      ]);
      setTempFile({ url: "", name: "" });
    },
    [helpers, meta.value],
  );

  const uploadToS3 = useCallback(
    async (uploadUrl: string, image: Image) => {
      try {
        const response = await fetch(image.path);
        const blob = await response.blob();

        await fetch(uploadUrl, {
          method: "PUT",
          body: blob,
          headers: {
            "Content-Type": blob.type,
          },
        });
        showMessage({ message: "Successfully uploaded", type: "success" });
        onSuccessfulUpload(uploadUrl, image);
      } catch (error) {
        showMessage({ message: "Something went wrong", type: "danger" });
      }
    },
    [onSuccessfulUpload],
  );

  const uploadImage = useCallback(
    async (image: Image) => {
      try {
        setLoading(true);
        const res: AwsResponseProp = await uploadADoc(
          image?.filename || "untitled",
        );
        await uploadToS3(res.putPresignedURL, image);
      } catch (e) {
        showMessage({ message: "Upload failed", type: "danger" });
      } finally {
        setLoading(false);
      }
    },
    [uploadADoc, uploadToS3],
  );

  const openActionSheet = useCallback(() => {
    const options = ["Open Gallery", "Open Camera", "Cancel"];
    const cancelButtonIndex = 2;

    showActionSheetWithOptions(
      {
        options,
        cancelButtonIndex,
      },
      (buttonIndex) => {
        if (buttonIndex === 0) {
          ImagePicker.openPicker(imageOption).then((image) => {
            setTempFile({ url: image.path, name: image.filename || "" });
            uploadImage(image);
          });
        } else if (buttonIndex === 1) {
          ImagePicker.openCamera(imageOption).then((image) => {
            setTempFile({ url: image.path, name: image.filename || "" });
          });
        }
      },
    );
  }, [showActionSheetWithOptions, uploadImage]);

  return (
    <View style={styles.container}>
      <View style={styles.row}>
        <Text style={styles.title}>{title}</Text>
        <Button
          style={styles.uploadButton}
          mode="outlined"
          icon={buttonLabel ? undefined : () => <Feather name={"upload"} />}
          onPress={openActionSheet}
          disabled={loading || disabled}
        >
          <Text variant={"bodyMedium"}>{buttonLabel || "Upload file"}</Text>
        </Button>
      </View>
      <View style={styles.flex}>
        {meta.value.map((m: FileDto) => (
          <Card key={m.resourceURL} style={styles.card} elevation={0}>
            <View style={styles.cardContent}>
              <View>
                <Text>{m.name}</Text>
                <Text style={styles.caption}>
                  {formatFileSize(m.size)} Completed
                </Text>
              </View>
              <View style={styles.iconRow}>
                <TouchableOpacity
                  onPress={() =>
                    helpers.setValue(addOrRemoveArr(meta.value, m))
                  }
                  disabled={loading || disabled}
                >
                  <MaterialCommunityIcons
                    name={"delete"}
                    color={"rgba(0, 0, 0, 0.56)"}
                    size={22}
                  />
                </TouchableOpacity>
                <AntDesign name={"checkcircle"} color={"#178D46"} size={22} />
              </View>
            </View>
          </Card>
        ))}
        {(tempFile?.name || tempFile?.url) && (
          <Card style={styles.card} elevation={0}>
            <View style={styles.cardContent}>
              <View>
                <Text>{`Attachment-${tempFile?.name}`}</Text>
                <Text style={styles.caption}>Uploading</Text>
              </View>
              <View>
                <ActivityIndicator color={"#178D46"} />
              </View>
            </View>
          </Card>
        )}
        {meta.error && <Text style={styles.errorText}>{meta.error}</Text>}
      </View>
    </View>
  );
};

const styles = StyleSheet.create({
  container: {
    width: "100%",
  },
  row: {
    flexDirection: "row",
    justifyContent: "space-between",
    alignItems: "center",
    flexWrap: "wrap",
  },
  title: {
    fontWeight: "500",
  },
  flex: {
    width: "100%",
    flexDirection: "column",
  },
  card: {
    width: "100%",
    padding: 10,
    justifyContent: "center",
    backgroundColor: "transparent",
    borderRadius: 4,
    borderWidth: 1,
    borderColor: "#DBDBDB",
    marginTop: 10,
  },
  cardContent: {
    flexDirection: "row",
    justifyContent: "space-between",
    alignItems: "center",
  },
  caption: {
    color: "gray",
  },
  iconRow: {
    flexDirection: "row",
    alignItems: "center",
  },
  errorText: {
    color: "red",
    marginTop: 10,
  },
  uploadButton: {
    borderRadius: 6,
  },
});
