import React, {
  useCallback,
  useContext,
  useEffect,
  useMemo,
  useState,
} from "react";
import axios, { type AxiosInstance } from "axios";
import type { IBusiness } from "../typings/business";
import analytics from "../utils/segment";
import { PaperProvider } from "react-native-paper";
import { darkTheme, lightTheme } from "../theme";
import type { ThemeProp } from "react-native-paper/src/types";
import FlashMessage from "react-native-flash-message";
import { SafeAreaProvider, SafeAreaView } from "react-native-safe-area-context";
import queryString from "query-string";
import { ActionSheetProvider } from "@expo/react-native-action-sheet";

type WithEmail = {
  email: string;
  contactNumber?: never;
  ntn?: never;
  external_user_id?: never;
};
type WithNtn = {
  email?: never;
  contactNumber?: never;
  ntn: string;
  external_user_id?: never;
};
type WithExternalUserId = {
  email?: never;
  contactNumber?: never;
  ntn?: never;
  external_user_id: string;
};
type WithContactNumber = {
  email?: never;
  contactNumber: string;
  ntn?: never;
  external_user_id?: never;
};

type LMSContextProps = {
  TOKEN: string;
  BASE_URL: string;
  axiosInstance: AxiosInstance;
  info: WithEmail | WithContactNumber | WithNtn | WithExternalUserId;
  setInfo?(
    info: WithEmail | WithContactNumber | WithNtn | WithExternalUserId,
  ): void;
  userDetail?: IBusiness;
};

type Props = {
  children: React.ReactNode;
  token: string;
  base_url: string;
  theme?: "light" | "dark";
  palette?: ThemeProp;
  info: WithEmail | WithContactNumber | WithNtn | WithExternalUserId;
  fonts?: string[];
};

const LMSContext = React.createContext<LMSContextProps>({} as LMSContextProps);
export const LMSProvider: React.FC<Props> = ({
  children,
  token,
  base_url,
  theme,
  palette,
  info,
}) => {
  const [userInfo, setUserInfo] = useState(info);
  const [userDetail, setUserDetail] = useState<IBusiness>();

  const axiosInstance = axios.create({
    baseURL: base_url,
    headers: {
      Authorization: `Bearer ${token}`,
    },
  });

  const fetchData = useCallback(async () => {
    try {
      const { data }: { data: IBusiness } = await axios.get(
        `${base_url}v1/external-lending/sme/users?${queryString.stringify(userInfo || {})}`,
        {
          headers: {
            Authorization: `Bearer ${token}`,
          },
        },
      );

      setUserDetail(data);
      analytics.identify(data.kyb.id);
    } catch (e: any) {
      //
    }
  }, [base_url, token, userInfo]);

  const value: LMSContextProps = useMemo(() => {
    return {
      TOKEN: token,
      BASE_URL: base_url,
      axiosInstance: axiosInstance,
      info: userInfo,
      setInfo: (
        inf: WithEmail | WithContactNumber | WithNtn | WithExternalUserId,
      ) => setUserInfo(inf),
      userDetail: userDetail,
    };
  }, [token, base_url, axiosInstance, userInfo, userDetail]);

  useEffect(() => {
    if (info) {
      fetchData();
    } else {
      setUserDetail(undefined);
    }
  }, [fetchData, info]);

  return (
    <ActionSheetProvider>
      <PaperProvider
        theme={palette ? palette : theme === "light" ? lightTheme : darkTheme}
      >
        <LMSContext.Provider value={value}>
          {children}
          <FlashMessage position="top" />
        </LMSContext.Provider>
      </PaperProvider>
    </ActionSheetProvider>
  );
};

export const useLMS = () => useContext(LMSContext);
